@@ -100,11 +100,11 @@ ephy_file_monitor_reload_cb (EphyFileMonitor *monitor)
 }
 
 static void
-ephy_file_monitor_changed_cb (GFileMonitor     *monitor,
-                              GFile            *file,
-                              GFile            *other_file,
-                              GFileMonitorEvent event_type,
-                              EphyFileMonitor  *file_monitor)
+ephy_file_monitor_changed_cb (GFileMonitor      *monitor,
+                              GFile             *file,
+                              GFile             *other_file,
+                              GFileMonitorEvent  event_type,
+                              EphyFileMonitor   *file_monitor)
 {
   gboolean should_reload;
 
@@ -275,8 +275,7 @@ ephy_file_monitor_class_init (EphyFileMonitorClass *klass)
 
   obj_properties[PROP_VIEW] =
     g_param_spec_object ("view",
-                         "View",
-                         "The file monitor's associated view",
+                         NULL, NULL,
                          EPHY_TYPE_WEB_VIEW,
                          G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS);
 
