@@ -21,7 +21,7 @@
 
 #pragma once
 
-#include <webkit2/webkit2.h>
+#include <webkit/webkit.h>
 
 #include "ephy-embed-shell.h"
 #include "ephy-history-types.h"
@@ -44,6 +44,7 @@ G_DECLARE_FINAL_TYPE (EphyWebView, ephy_web_view, EPHY, WEB_VIEW, WebKitWebView)
                                         "^inspector://.*$|" \
                                         "^webkit://.*$|" \
                                         "^ephy-resource://.*$|" \
+                                        "^view-source:.*$|" \
                                         "^ephy-reader:.*$" \
                                         ")"
 
@@ -61,7 +62,6 @@ typedef enum
   EPHY_WEB_VIEW_DOCUMENT_HTML,
   EPHY_WEB_VIEW_DOCUMENT_XML,
   EPHY_WEB_VIEW_DOCUMENT_IMAGE,
-  EPHY_WEB_VIEW_DOCUMENT_PDF,
   EPHY_WEB_VIEW_DOCUMENT_OTHER
 } EphyWebViewDocumentType;
 
@@ -76,8 +76,6 @@ typedef enum {
   EPHY_WEB_VIEW_ERROR_NO_SUCH_FILE,
 } EphyWebViewErrorPage;
 
-typedef void (* EphyPasswordSaveRequestCallback) (int      response_id,
-                                                  gpointer data);
 
 GType                      ephy_web_view_chrome_get_type          (void);
 GType                      ephy_web_view_security_level_get_type  (void);
@@ -89,7 +87,7 @@ void                       ephy_web_view_load_url                 (EphyWebView
                                                                    const char                *url);
 gboolean                   ephy_web_view_is_loading               (EphyWebView               *view);
 gboolean                   ephy_web_view_load_failed              (EphyWebView               *view);
-GdkPixbuf *                ephy_web_view_get_icon                 (EphyWebView               *view);
+GIcon *                    ephy_web_view_get_icon                 (EphyWebView               *view);
 EphyWebViewDocumentType    ephy_web_view_get_document_type        (EphyWebView               *view);
 EphyWebViewNavigationFlags ephy_web_view_get_navigation_flags     (EphyWebView               *view);
 const char *               ephy_web_view_get_status_message       (EphyWebView               *view);
@@ -105,6 +103,7 @@ gboolean            ephy_web_view_get_should_bypass_safe_browsing (EphyWebView
 void                ephy_web_view_set_should_bypass_safe_browsing (EphyWebView               *view,
                                                                    gboolean                   bypass_safe_browsing);
 gboolean                   ephy_web_view_get_is_blank             (EphyWebView               *view);
+gboolean                   ephy_web_view_is_newtab                (EphyWebView               *view);
 gboolean                   ephy_web_view_is_overview              (EphyWebView               *view);
 void                       ephy_web_view_has_modified_forms       (EphyWebView               *view,
                                                                    GCancellable              *cancellable,
@@ -176,13 +175,6 @@ gboolean                   ephy_web_view_get_reader_mode_state    (EphyWebView
 
 gboolean                   ephy_web_view_is_in_auth_dialog        (EphyWebView               *view);
 
-void                       ephy_web_view_show_auth_form_save_request (EphyWebView                    *web_view,
-                                                                      const char                     *origin,
-                                                                      const char                     *username,
-                                                                      EphyPasswordSaveRequestCallback response_callback,
-                                                                      gpointer                        response_data,
-                                                                      GDestroyNotify                  response_destroy);
-
 GtkWidget                 *ephy_web_view_new_with_user_content_manager (WebKitUserContentManager *ucm);
 
 guint64                    ephy_web_view_get_uid                       (EphyWebView *web_view);
