@@ -1,5 +1,7 @@
 'use strict';
 
+/* exported contextMenuElementIsPassword, contextMenuElementIsEditable */
+
 // eslint-disable-next-line no-var
 var Ephy = {};
 
@@ -472,6 +474,7 @@ Ephy.FormManager = class FormManager
         this._passwordFormMessageSerializer = null;
         this._preFillUserMenu = null;
         this._elementBeingAutoFilled = null;
+        this._submissionHandled = false;
     }
 
     // Public
@@ -567,6 +570,11 @@ Ephy.FormManager = class FormManager
         if (!Ephy.shouldRememberPasswords())
             return;
 
+        if (this._submissionHandled)
+            return;
+
+        this._submissionHandled = true;
+
         const formAuth = this._generateFormAuth(false);
         if (!formAuth || !formAuth.password)
             return;
@@ -792,3 +800,12 @@ Ephy.FormManager = class FormManager
         return formAuth;
     }
 };
+
+let contextMenuElementIsEditable = false;
+let contextMenuElementIsPassword = false;
+
+window.document.addEventListener('contextmenu', (event) => {
+    // isContentEditable is always false, in practice this seems functional enough.
+    contextMenuElementIsEditable = event.target.tagName.toLowerCase() === 'input';
+    contextMenuElementIsPassword = event.target.type === 'password';
+});
