@@ -20,6 +20,7 @@
  *  along with Epiphany.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include "config.h"
 #include "prefs-privacy-page.h"
 
 #include "ephy-settings.h"
@@ -33,17 +34,19 @@ enum {
 };
 
 struct _PrefsPrivacyPage {
-  HdyPreferencesPage parent_instance;
+  AdwPreferencesPage parent_instance;
 
-  /* Web Content */
-  GtkWidget *popups_allow_switch;
-  GtkWidget *adblock_allow_switch;
+  /* Web Safety */
+  GtkWidget *safe_browsing_group;
   GtkWidget *enable_safe_browsing_switch;
 
-  /* Cookies */
-  GtkWidget *always;
-  GtkWidget *no_third_party;
-  GtkWidget *never;
+  /* Web Tracking */
+  GtkWidget *enable_itp_switch;
+  GtkWidget *enable_website_data_storage_switch;
+
+  /* Search Suggestions */
+  GtkWidget *search_suggestions_box;
+  GtkWidget *enable_google_search_suggestions_switch;
 
   /* Passwords */
   GtkWidget *remember_passwords_switch;
@@ -51,7 +54,7 @@ struct _PrefsPrivacyPage {
 
 static guint signals[LAST_SIGNAL];
 
-G_DEFINE_TYPE (PrefsPrivacyPage, prefs_privacy_page, HDY_TYPE_PREFERENCES_PAGE)
+G_DEFINE_TYPE (PrefsPrivacyPage, prefs_privacy_page, ADW_TYPE_PREFERENCES_PAGE)
 
 static void
 on_passwords_row_activated (GtkWidget        *row,
@@ -67,67 +70,14 @@ on_clear_data_row_activated (GtkWidget        *row,
   g_signal_emit (privacy_page, signals[CLEAR_DATA_ROW_ACTIVATED], 0);
 }
 
-static gboolean
-cookies_get_mapping (GValue   *value,
-                     GVariant *variant,
-                     gpointer  user_data)
-{
-  const char *setting;
-  const char *name;
-
-  setting = g_variant_get_string (variant, NULL);
-  name = gtk_buildable_get_name (GTK_BUILDABLE (user_data));
-
-  if (g_strcmp0 (name, "no_third_party") == 0)
-    name = "no-third-party";
-
-  /* If the button name matches the setting, it should be active. */
-  if (g_strcmp0 (name, setting) == 0)
-    g_value_set_boolean (value, TRUE);
-
-  return TRUE;
-}
-
-static GVariant *
-cookies_set_mapping (const GValue       *value,
-                     const GVariantType *expected_type,
-                     gpointer            user_data)
-{
-  GVariant *variant = NULL;
-  const char *name;
-
-  /* Don't act unless the button has been activated (turned ON). */
-  if (!g_value_get_boolean (value))
-    return NULL;
-
-  name = gtk_buildable_get_name (GTK_BUILDABLE (user_data));
-  if (g_strcmp0 (name, "no_third_party") == 0)
-    variant = g_variant_new_string ("no-third-party");
-  else
-    variant = g_variant_new_string (name);
-
-  return variant;
-}
-
 static void
 setup_privacy_page (PrefsPrivacyPage *privacy_page)
 {
   GSettings *web_settings = ephy_settings_get (EPHY_PREFS_WEB_SCHEMA);
 
   /* ======================================================================== */
-  /* ========================== Web Content ================================= */
+  /* ========================== Web Safety ================================== */
   /* ======================================================================== */
-  g_settings_bind (web_settings,
-                   EPHY_PREFS_WEB_ENABLE_ADBLOCK,
-                   privacy_page->adblock_allow_switch,
-                   "active",
-                   G_SETTINGS_BIND_DEFAULT);
-
-  g_settings_bind (web_settings,
-                   EPHY_PREFS_WEB_ENABLE_POPUPS,
-                   privacy_page->popups_allow_switch,
-                   "active",
-                   G_SETTINGS_BIND_INVERT_BOOLEAN);
 
   g_settings_bind (web_settings,
                    EPHY_PREFS_WEB_ENABLE_SAFE_BROWSING,
@@ -135,38 +85,23 @@ setup_privacy_page (PrefsPrivacyPage *privacy_page)
                    "active",
                    G_SETTINGS_BIND_DEFAULT);
 
-  /* ======================================================================== */
-  /* ========================== Cookies ===================================== */
-  /* ======================================================================== */
-  g_settings_bind_with_mapping (web_settings,
-                                EPHY_PREFS_WEB_COOKIES_POLICY,
-                                privacy_page->always,
-                                "active",
-                                G_SETTINGS_BIND_DEFAULT,
-                                cookies_get_mapping,
-                                cookies_set_mapping,
-                                privacy_page->always,
-                                NULL);
+  gtk_widget_set_visible (privacy_page->safe_browsing_group, ENABLE_GSB);
 
-  g_settings_bind_with_mapping (web_settings,
-                                EPHY_PREFS_WEB_COOKIES_POLICY,
-                                privacy_page->no_third_party,
-                                "active",
-                                G_SETTINGS_BIND_DEFAULT,
-                                cookies_get_mapping,
-                                cookies_set_mapping,
-                                privacy_page->no_third_party,
-                                NULL);
+  /* ======================================================================== */
+  /* ========================== Web Tracking ================================ */
+  /* ======================================================================== */
 
-  g_settings_bind_with_mapping (web_settings,
-                                EPHY_PREFS_WEB_COOKIES_POLICY,
-                                privacy_page->never,
-                                "active",
-                                G_SETTINGS_BIND_DEFAULT,
-                                cookies_get_mapping,
-                                cookies_set_mapping,
-                                privacy_page->never,
-                                NULL);
+  g_settings_bind (web_settings,
+                   EPHY_PREFS_WEB_ENABLE_ITP,
+                   privacy_page->enable_itp_switch,
+                   "active",
+                   G_SETTINGS_BIND_DEFAULT);
+
+  g_settings_bind (web_settings,
+                   EPHY_PREFS_WEB_ENABLE_WEBSITE_DATA_STORAGE,
+                   privacy_page->enable_website_data_storage_switch,
+                   "active",
+                   G_SETTINGS_BIND_DEFAULT);
 
   /* ======================================================================== */
   /* ========================== Passwords =================================== */
@@ -176,6 +111,15 @@ setup_privacy_page (PrefsPrivacyPage *privacy_page)
                    privacy_page->remember_passwords_switch,
                    "active",
                    G_SETTINGS_BIND_DEFAULT);
+
+  /* ======================================================================== */
+  /* ========================== Search Suggestions ========================== */
+  /* ======================================================================== */
+  g_settings_bind (EPHY_SETTINGS_MAIN,
+                   EPHY_PREFS_USE_GOOGLE_SEARCH_SUGGESTIONS,
+                   privacy_page->enable_google_search_suggestions_switch,
+                   "active",
+                   G_SETTINGS_BIND_DEFAULT);
 }
 
 static void
@@ -200,15 +144,17 @@ prefs_privacy_page_class_init (PrefsPrivacyPageClass *klass)
                   0, NULL, NULL, NULL,
                   G_TYPE_NONE, 0);
 
-  /* Web Content */
-  gtk_widget_class_bind_template_child (widget_class, PrefsPrivacyPage, popups_allow_switch);
-  gtk_widget_class_bind_template_child (widget_class, PrefsPrivacyPage, adblock_allow_switch);
+  /* Web Safety */
+  gtk_widget_class_bind_template_child (widget_class, PrefsPrivacyPage, safe_browsing_group);
   gtk_widget_class_bind_template_child (widget_class, PrefsPrivacyPage, enable_safe_browsing_switch);
 
-  /* Cookies */
-  gtk_widget_class_bind_template_child (widget_class, PrefsPrivacyPage, always);
-  gtk_widget_class_bind_template_child (widget_class, PrefsPrivacyPage, no_third_party);
-  gtk_widget_class_bind_template_child (widget_class, PrefsPrivacyPage, never);
+  /* Web Tracking */
+  gtk_widget_class_bind_template_child (widget_class, PrefsPrivacyPage, enable_itp_switch);
+  gtk_widget_class_bind_template_child (widget_class, PrefsPrivacyPage, enable_website_data_storage_switch);
+
+  /* Search Suggestions */
+  gtk_widget_class_bind_template_child (widget_class, PrefsPrivacyPage, search_suggestions_box);
+  gtk_widget_class_bind_template_child (widget_class, PrefsPrivacyPage, enable_google_search_suggestions_switch);
 
   /* Passwords */
   gtk_widget_class_bind_template_child (widget_class, PrefsPrivacyPage, remember_passwords_switch);
@@ -221,7 +167,12 @@ prefs_privacy_page_class_init (PrefsPrivacyPageClass *klass)
 static void
 prefs_privacy_page_init (PrefsPrivacyPage *privacy_page)
 {
+  EphyEmbedShellMode mode = ephy_embed_shell_get_mode (ephy_embed_shell_get_default ());
+
   gtk_widget_init_template (GTK_WIDGET (privacy_page));
 
   setup_privacy_page (privacy_page);
+
+  gtk_widget_set_visible (privacy_page->search_suggestions_box,
+                          mode != EPHY_EMBED_SHELL_MODE_APPLICATION);
 }
