@@ -23,6 +23,7 @@
 #include "ephy-search-provider.h"
 
 #include "ephy-bookmarks-manager.h"
+#include "ephy-embed-utils.h"
 #include "ephy-file-helpers.h"
 #include "ephy-prefs.h"
 #include "ephy-profile-utils.h"
@@ -48,10 +49,10 @@ struct _EphySearchProvider {
 };
 
 struct _EphySearchProviderClass {
-  GApplicationClass parent_class;
+  EphyEmbedShellClass parent_class;
 };
 
-G_DEFINE_TYPE (EphySearchProvider, ephy_search_provider, G_TYPE_APPLICATION)
+G_DEFINE_TYPE (EphySearchProvider, ephy_search_provider, EPHY_TYPE_EMBED_SHELL)
 
 #define INACTIVITY_TIMEOUT 60 * 1000 /* One minute, in milliseconds */
 
@@ -115,6 +116,7 @@ gather_results_async (EphySearchProvider   *self,
 
   ephy_suggestion_model_query_async (self->model,
                                      search_string,
+                                     FALSE,
                                      cancellable,
                                      (GAsyncReadyCallback)on_model_updated,
                                      task);
@@ -167,6 +169,22 @@ handle_get_subsearch_result_set (EphyShellSearchProvider2  *skeleton,
   g_application_hold (G_APPLICATION (self));
   g_cancellable_reset (self->cancellable);
 
+  if (g_strv_length (terms) == 1) {
+    gboolean is_uri = ephy_embed_utils_address_is_valid (terms[0]);
+
+    if (is_uri) {
+      GPtrArray *results = g_ptr_array_new ();
+
+      g_ptr_array_add (results, g_strdup_printf ("special:load:%s", terms[0]));
+      g_ptr_array_add (results, NULL);
+      g_dbus_method_invocation_return_value (invocation,
+                                             g_variant_new ("(^as)", g_ptr_array_free (results, FALSE)));
+
+      g_application_release (G_APPLICATION (self));
+      return TRUE;
+    }
+  }
+
   gather_results_async (self, terms, self->cancellable,
                         complete_request, invocation);
 
@@ -198,6 +216,16 @@ handle_get_result_metas (EphyShellSearchProvider2  *skeleton,
       g_variant_builder_add (&builder, "{sv}",
                              "gicon", g_variant_new_string (APPLICATION_ID));
       g_variant_builder_close (&builder);
+    } else if (g_str_has_prefix (results[i], "special:load:")) {
+      g_variant_builder_open (&builder, G_VARIANT_TYPE ("a{sv}"));
+      g_variant_builder_add (&builder, "{sv}",
+                             "id", g_variant_new_string ("special:load"));
+      g_variant_builder_add (&builder, "{sv}",
+                             "name", g_variant_new_take_string (g_strdup_printf (_("Load “%s”"),
+                                                                                 results[i] + strlen ("special:load:"))));
+      g_variant_builder_add (&builder, "{sv}",
+                             "gicon", g_variant_new_string (APPLICATION_ID));
+      g_variant_builder_close (&builder);
     } else {
       EphySuggestion *suggestion;
       const char *title;
@@ -249,25 +277,10 @@ launch_search (EphySearchProvider  *self,
                guint                timestamp)
 {
   g_autofree char *search_string = NULL;
-  g_autofree char *query_param = NULL;
   g_autofree char *effective_url = NULL;
-  const char *address_search;
-  EphyEmbedShell *shell;
-  EphySearchEngineManager *search_engine_manager;
-
-  shell = ephy_embed_shell_get_default ();
-  search_engine_manager = ephy_embed_shell_get_search_engine_manager (shell);
-  address_search = ephy_search_engine_manager_get_default_search_address (search_engine_manager);
 
   search_string = g_strjoinv (" ", terms);
-  query_param = soup_form_encode ("q", search_string, NULL);
-
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wformat-nonliteral"
-  /* Format string under control of user input... but gsettings is trusted input. */
-  /* + 2 here is getting rid of 'q=' */
-  effective_url = g_strdup_printf (address_search, query_param + 2);
-#pragma GCC diagnostic pop
+  effective_url = ephy_embed_utils_autosearch_address (search_string);
 
   launch_uri (effective_url, timestamp);
 }
@@ -285,6 +298,8 @@ handle_activate_result (EphyShellSearchProvider2  *skeleton,
 
   if (strcmp (identifier, "special:search") == 0)
     launch_search (self, terms, timestamp);
+  else if (strcmp (identifier, "special:load") == 0)
+    launch_uri (terms[0], timestamp);
   else
     launch_uri (identifier, timestamp);
 
@@ -315,14 +330,12 @@ handle_launch_search (EphyShellSearchProvider2  *skeleton,
 static void
 ephy_search_provider_init (EphySearchProvider *self)
 {
-  g_autofree char *filename = NULL;
   EphyEmbedShell *shell = ephy_embed_shell_get_default ();
 
   g_application_set_flags (G_APPLICATION (self), G_APPLICATION_IS_SERVICE);
 
   self->settings = g_settings_new (EPHY_PREFS_SCHEMA);
 
-  filename = g_build_filename (ephy_profile_dir (), EPHY_HISTORY_FILE, NULL);
   self->bookmarks_manager = ephy_bookmarks_manager_new ();
   self->model = ephy_suggestion_model_new (ephy_embed_shell_get_global_history_service (shell),
                                            self->bookmarks_manager);
@@ -418,5 +431,6 @@ ephy_search_provider_new (void)
 
   return g_object_new (EPHY_TYPE_SEARCH_PROVIDER,
                        "application-id", app_id,
+                       "mode", EPHY_EMBED_SHELL_MODE_SEARCH_PROVIDER,
                        NULL);
 }
