@@ -1774,7 +1774,7 @@ set_default_application_title (EphyApplicationDialogData *data,
     g_autoptr (GUri) uri = NULL;
     const char *host;
 
-    uri = g_uri_parse (webkit_web_view_get_uri (WEBKIT_WEB_VIEW (data->view)), G_URI_FLAGS_NONE, NULL);
+    uri = g_uri_parse (webkit_web_view_get_uri (WEBKIT_WEB_VIEW (data->view)), G_URI_FLAGS_PARSE_RELAXED, NULL);
     host = g_uri_get_host (uri);
 
     if (host != NULL && host[0] != '\0') {
@@ -2011,7 +2011,7 @@ get_suggested_filename (EphyEmbed  *embed,
   web_resource = webkit_web_view_get_main_resource (WEBKIT_WEB_VIEW (view));
   response = webkit_web_resource_get_response (web_resource);
   mimetype = webkit_uri_response_get_mime_type (response);
-  uri = g_uri_parse (webkit_web_resource_get_uri (web_resource), G_URI_FLAGS_SCHEME_NORMALIZE, NULL);
+  uri = g_uri_parse (webkit_web_resource_get_uri (web_resource), G_URI_FLAGS_PARSE_RELAXED | G_URI_FLAGS_SCHEME_NORMALIZE, NULL);
   page_title = ephy_embed_get_title (embed);
   filename = g_strconcat (page_title, file_extension, NULL);
 
